import {getElements, getOneElementOrThrow} from "../libs/element.js";
import {addFavourite, getFavourites, removeFavouriteByIndex} from "../libs/config.js";
import {getTemplate} from "../libs/template.js";
import {navigate} from "../libs/navigation.js";
import {ask, open} from "../libs/dialog.js";
import {basename} from "../libs/path.js";

export const renderFavourite = () => {
    const favouriteList = getOneElementOrThrow(document, '[data-ui=favourites]');

    const addedItems = getElements(favouriteList, '[data-el=added]');
    addedItems.forEach(addedItems => addedItems.remove());

    const favourites = getFavourites();
    favourites.forEach(({ name, path, system }, index) => {
        const template = getTemplate('favourites');
        const actionEl = template.el('action');
        actionEl.textContent = name;
        actionEl.addEventListener('click', () => navigate(path));

        const removeEl = template.el('remove');
        if (system) {
            removeEl.remove();
        } else {
            removeEl.addEventListener('click', async () => {
                const confirm = await ask(`Remove ${name} from favourites?`);
                if (confirm) {
                    removeFavouriteByIndex(index);

                    renderFavourite();
                }
            });
        }

        favouriteList.appendChild(template.render());
    });

    const addEl = getOneElementOrThrow(document, '[data-ui=add-favourite]');

    /**
     * @type {HTMLDialogElement}
     */
    const dialog = getOneElementOrThrow(document, '[data-ui=favourite-dialog]');
    dialog.close();
    addEl.addEventListener('click', async () => {
        const nameEl = getOneElementOrThrow(dialog, '[data-ui=favourite-name]');
        const submitEl = getOneElementOrThrow(dialog, '[data-ui=favourite-submit]');
        const closeEl = getOneElementOrThrow(dialog, '[data-ui=favourite-close]');

        const path = await open({ directory: true })
        nameEl.value = await basename(path);

        dialog.showModal();

        const handleClose = () => {
            nameEl.value = '';
            dialog.close();
        }

        closeEl.addEventListener('click', () => handleClose());

        submitEl.addEventListener('click', () => {
            const { value: name } = nameEl;

            if (name.trim() && path.trim()) {
                const favourites = getFavourites();

                addFavourite({ name, path, index: favourites.length });

                renderFavourite();

                handleClose();
            }
        });
    });
}